/**
 * Copyright (c) 2004-2007 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common;

/*
 * 2007-07-10 [KS]  Package Umstrukturierung nach Segment - SWE (=> funclib),
 *                  Zeitberechnungen auf "IntervalHelper" umgestellt (alte
 *                  Klassen Intervalle+Constants wurden entfernt) und neue
 *                  Konstanten entsprechend eingefügt
 * [KS] 2006-11-27  komplette Umstellung der Datum/Zeitberechnung auf Grund
 *                  von Fehlern bei der Sommerzeitumstellung * Optimierung der
 *                  Geschwindigkeit * zusätzliche Funktionen zur Berechnung der
 *                  Anzahl Intervalle zwischen 2 Datum/Zeitangaben * Ergänzung
 *                  von Kommentaren
 *
 * Revision 1.3  2007/08/22 07:48:46  Trier
 * <Kein Kommentar eingegeben>
 */

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * <p>Klasse die statische Methoden zum Arbeiten mit Intervallen liefert.</p>
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: IntervalHelper.java 442 2008-11-10 06:59:32Z Schawaller $
 */
abstract public class IntervalHelper {

  public static final long TAG_IN_MS = 86400000;
  public static final long STUNDE_IN_MS = 3600000;
  public static final long MINUTE_IN_MS = 60000;
  public static final long SEKUNDE_IN_MS = 1000;

  private IntervalHelper() {}

  /**
   * <p>Bestimmt, ob die übergebene Zeit gültig für den angegebenen
   * IntervallTyp ist.</p>
   *
   * @param intervalType {@link IntervalType}
   * @param time long
   * @return boolean
   */
  public static boolean isValid(IntervalType intervalType, long time) {
    return (getPrecision(time).compareTo(intervalType) >= 0);
  }

  /**
   * <p>Liefert den zu Datum/Zeit passenden IntervalType. Die Ermittlung
   * erfolgt darüber, wie die Angabe gerundet ist.</p>
   * @param time long
   * @return {@link IntervalType}
   */
  private static IntervalType getPrecision(long time) {
    Calendar cal = Calendar.getInstance();
    cal.setTimeInMillis(time);
    if (cal.get(Calendar.SECOND) != 0) {
      return IntervalType.SECOND;
    }
    if (cal.get(Calendar.MINUTE) != 0) {
      return IntervalType.MINUTE;
    }
    if (cal.get(Calendar.HOUR) != 0) {
      return IntervalType.HOUR;
    }
    if (cal.get(Calendar.DAY_OF_MONTH) != 1) {
      return IntervalType.DAY;
    }
    if (cal.get(Calendar.MONTH) != Calendar.JANUARY) {
      return IntervalType.MONTH;
    }
    return IntervalType.YEAR;
  }

  /**
   * <p>Liefert den Anfangszeitpunkt des Intervalles, in das der angegebene
   * Zeitpunkt fällt.</p>
   *
   * @param intervalType {@link IntervalType}
   * @param cal {@link Calendar} zu rundende Datum/Zeitangabe
   * @return {@link Calendar} auf Intervallanfang gerundete Datum/Zeitangabe
   */
  public static Calendar getIntervalBegin( IntervalType intervalType, Calendar cal )
  {
    Calendar c;
    switch ( intervalType ) {
      case YEAR:
        return new GregorianCalendar( cal.get( Calendar.YEAR ), 0, 1 );
      case MONTH:
        return new GregorianCalendar( cal.get( Calendar.YEAR ), cal.get( Calendar.MONTH ), 1 );
      case DAY:
        return new GregorianCalendar( cal.get( Calendar.YEAR ), cal.get( Calendar.MONTH ),
                                      cal.get( Calendar.DAY_OF_MONTH ) );
      case HOUR:
        c = ( Calendar ) ( cal.clone() );
        c.setTimeInMillis( ( c.getTimeInMillis() / STUNDE_IN_MS ) * STUNDE_IN_MS );
        return c;
      case MINUTE:
        c = ( Calendar ) ( cal.clone() );
        c.setTimeInMillis( ( c.getTimeInMillis() / MINUTE_IN_MS ) * MINUTE_IN_MS );
        return c;
    }
    return cal;

  }

  /**
   * <p>Liefert den Anfangszeitpunkt des Intervalles, in das der angegebene
   * Zeitpunkt fällt.</p>
   *
   * @param intervalType {@link IntervalType}
   * @param date {@link Date} zu rundendes Datum
   * @return {@link Date} auf Intervallanfang gerundete Datum/Zeitangabe
   */
  public static Date getIntervalBegin( IntervalType intervalType, Date date )
  {
    if ( intervalType == IntervalType.HOUR ) {
      return new Date( ( date.getTime() / STUNDE_IN_MS ) * STUNDE_IN_MS );
    } else if ( intervalType == IntervalType.MINUTE ) {
        return new Date( ( date.getTime() / MINUTE_IN_MS ) * MINUTE_IN_MS );
    } else {
      Calendar cal = GregorianCalendar.getInstance();
      cal.setTime( date );
      return getIntervalBegin( intervalType,cal ).getTime();
    }
  }

  /**
   * <p>Liefert den Anfangszeitpunkt des Intervalles, in das der angegebene
   * Zeitpunkt fällt.</p>
   *
   * @param intervalType {@link IntervalType}
   * @param time <code>long</code> zu rundendes Datum als UTC Angabe in ms
   * @return <code>long</code> auf Intervallanfang gerundete Datum/Zeitangabe
   */
  public static long getIntervalBegin(IntervalType intervalType, long time) {
    if ( intervalType == IntervalType.HOUR ) {
      return ( time / STUNDE_IN_MS ) * STUNDE_IN_MS;
    } else if ( intervalType == IntervalType.MINUTE ) {
      return ( time / MINUTE_IN_MS ) * MINUTE_IN_MS;
    } else {
      Calendar cal = GregorianCalendar.getInstance();
      cal.setTimeInMillis( time );
      return getIntervalBegin( intervalType,cal ).getTimeInMillis();
    }
  }

  /**
   * <p>Berechnung des nächstfolgenden Intervallanfangs auf Basis des
   * angegebenen Datums und der durch die Intervallart vorgegebenen
   * Intervalllänge.</p>
   * <p>Zur Berechnung wird die Intervalllänge korrekt dazugerechnet und
   * anschließend auf den Intervallbeginn gerundet. Um bei Intervallart Stunde
   * auch zum Zeitpunkt der Sommerzeitumstellung korrekte Werte zu erhalten
   * wird direkt über die Millisekunden gearbeitet. Ansonsten wird der Wechsel
   * über die Intervallgrenzen mit Hilfe der Kalenderfunktionen korrekt
   * ermittelt: bei Intervallart {@link Calendar#MONTH} und {@link Date} =
   * 2004-02-01 00°° wird das nächste Intervall korrekt mit 2004-03-01 00°°
   * berechnet.</p>
   *
   * @param intervalType {@link IntervalType} Intervallart/-länge
   * @param cal {@link Calendar} Ausgangsdatum
   * @param cnt <code>int</code> Anzahl Intervalle
   * @return {@link Calendar} Intervallbeginn des Folgeintervalls
   */
  public static Calendar getNextIntervalBegin( IntervalType intervalType, Calendar cal, int cnt )
  {
    Calendar calNext = ( Calendar )cal.clone();
    switch ( intervalType ) {
      case YEAR:
        calNext.add( Calendar.YEAR, cnt );
        break;
      case MONTH:
        calNext.add( Calendar.MONTH, cnt );
        break;
      case DAY:
        calNext.add( Calendar.DATE, cnt );
        break;
      case HOUR:
        calNext.setTimeInMillis( calNext.getTimeInMillis() + cnt * STUNDE_IN_MS );
        break;
      case MINUTE:
        calNext.setTimeInMillis( calNext.getTimeInMillis() + cnt * MINUTE_IN_MS );
        break;
    }
    return getIntervalBegin( intervalType, calNext );
  }

  /**
   * <p>Berechnung des nächstfolgenden Intervallanfangs auf Basis des
   * angegebenen Datums und der durch die Intervallart vorgegebenen
   * Intervalllänge.</p>
   *
   * @param intervall Intervallart/-länge
   * @param cal Ausgangsdatum
   * @return Intervallbeginn des Folgeintervalls
   */
  public static Calendar getNextIntervalBegin( IntervalType intervall, Calendar cal )
  {
    return getNextIntervalBegin( intervall, cal, 1 );
  }

  /**
   * <p>Berechnung des nächstfolgenden Intervallanfangs auf Basis des
   * angegebenen Datums und der durch die Intervallart vorgegebenen
   * Intervalllänge sowie anzahl Intervalle.</p>
   *
   * @param intervalType {@link IntervalType} Intervallart/-länge
   * @param time Ausgangsdatum
   * @param cnt <code>int</code> anzahl Intervalle
   * @return {@link Calendar} Intervallbeginn des Folgeintervalls
   */
  public static long getNextIntervalBegin( IntervalType intervalType, long time, int cnt )
  {
    if ( intervalType == IntervalType.HOUR ) {
      return ( ( time / STUNDE_IN_MS ) + cnt ) * STUNDE_IN_MS;
    } else if ( intervalType == IntervalType.MINUTE ) {
      return ( ( time / MINUTE_IN_MS ) + cnt ) * MINUTE_IN_MS;
    } else {
      Calendar cal = GregorianCalendar.getInstance();
      cal.setTimeInMillis( time );
      return getNextIntervalBegin( intervalType, cal, cnt ).getTimeInMillis();
    }
  }

  /**
   * <p>Berechnung des nächstfolgenden Intervallanfangs auf Basis des
   * angegebenen Datums und der durch die Intervallart vorgegebenen
   * Intervalllänge.</p>
   *
   * @param time <code>long</code> Ausgangsdatum
   * @param intervalType {@link IntervalType} Intervallart/-länge
   * @return <code>long</code> Intervallbeginn des Folgeintervalls
   */
  public static long getNextIntervalBegin( IntervalType intervalType, long time )
  {
    return getNextIntervalBegin( intervalType, time, 1 );
  }

  /**
   * <p>Berechnung die Anzahl Intervalle zwischen zwei Datum/Zeitangaben
   * auf Basis der durch die Intervallart vorgegebenen Intervalllänge.</p>
   *
   * @param from {@link Calendar} Ausgangsdatum (von)
   * @param to {@link Calendar} Ausgangsdatum (bis)
   * @param intervalType {@link IntervalType} Intervallart/-länge
   * @return {@link Calendar} Intervallbeginn des Folgeintervalls
   */
  public static int getAnzahlIntervalle( IntervalType intervalType, Calendar from, Calendar to )
  {
    Calendar f = ( from.getTimeInMillis() < to.getTimeInMillis() )
        ? getIntervalBegin( intervalType, from ) : getIntervalBegin( intervalType, to );
    Calendar t = ( from.getTimeInMillis() < to.getTimeInMillis() )
        ? getIntervalBegin( intervalType, to ) : getIntervalBegin( intervalType, from );
    int cnt = 1;
    switch ( intervalType ) {
      case HOUR:
        cnt += ( t.getTimeInMillis() - f.getTimeInMillis() ) / STUNDE_IN_MS;
        break;
      case DAY:
        // [KS] hier ist mir noch nichts zur direkten Berechnung eingefallen
        for ( int year = f.get( Calendar.YEAR ); year < t.get( Calendar.YEAR ); year++ ) {
          cnt += ( ( ( year % 4 == 0 ) && ! ( year % 100 == 0 ) ) || ( year % 400 == 0 ) ) ? 366 :
              365;
        }
        cnt += t.get( Calendar.DAY_OF_YEAR ) - f.get( Calendar.DAY_OF_YEAR );
        break;
      case MONTH:
        cnt += ( t.get( Calendar.YEAR ) * 12 + t.get( Calendar.MONTH ) ) -
            ( f.get( Calendar.YEAR ) * 12 + f.get( Calendar.MONTH ) );
        break;
      case YEAR:
        cnt += t.get( Calendar.YEAR ) - f.get( Calendar.YEAR );
        break;
    }
    return cnt;
  }

  /**
   * <p>Berechnung die Anzahl Intervalle zwischen zwei Datum/Zeitangaben
   * auf Basis der durch die Intervallart vorgegebenen Intervalllänge.</p>
   *
   * @param from <code>long</code> Ausgangsdatum (von),
   *   (<code>Calendar.getTimeInMillis()</code>)
   * @param to <code>long</code> Ausgangsdatum (bis),
   *   (<code>Calendar.getTimeInMillis()</code>)
   * @param intervalType {@link IntervalType} Intervallart/-länge
   * @return {@link Calendar} Intervallbeginn des Folgeintervalls
   */
  public static int getAnzahlIntervalle( long from, long to, IntervalType intervalType )
  {
    Calendar f = new GregorianCalendar();
    f.setTimeInMillis( from );
    Calendar t = new GregorianCalendar();
    t.setTimeInMillis( to );
    return getAnzahlIntervalle( intervalType, f, t );
  }

}
