/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.bast.common;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;
import de.bsvrz.sys.funclib.debug.Debug;

import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.*;

//~ KLASSEN ===================================================================

/**
 * <p>Klasse zum Lesen von Daten f&uuml;r eine Reihe von Zeitpunkten innerhalb
 * eines Zeitraumes aus dem Archiv und die weitere Verarbeitung durch einen oder
 * mehrere {@link ResultDataChannel}(s).</p>
 *
 * <p>In der Methode {@link #processData()} wird für alle Zeitpunkte die
 * aktuell g&uuml;tigen Daten der Kombinationen SystemObject/DataDescription
 * eines Zeitpunktes als ResultData[] an den oder die {@link ResultDataChannel
 * ResultDataChannel(s)} &uuml;bergeben.</p>
 *
 * <p>Die zu verarbeitenden Kombinationen aus SystemObject und DataDescription
 * werden durch die Methode
 * {@link #addDataCombination(SystemObject so, DataDescription dataDescription) addDataKombination}
 * festgelegt.</p> Die
 * Reihenfolge im an den/die Channel &uuml;bergebenen DataResult[] entspricht
 * der Reihenfolge der Methodenaufrufe
 * {@link #addDataCombination(SystemObject so, DataDescription dataDescription)}. <pre>
 *   Die Zeitpunkte wird bestimmt durch:
 *     1. Zeitraum (Period) - Periodenanfang, Periodenl&auml;nge, Einheit der Periodenl&auml;nge
 *     2. Schrittweite (Increment) - IncrementWert, IncrementEinheit
 *   </pre>
 *
 * <p>Der erste Zeitpunkt ist gleich dem Periodenanfang, der letzte ist kleiner als Periodenanfang + Periodenl&auml;nge.<pre>
 *  Beispiel f&uuml;r die Berechnung der Zeitpunkte:
 *  <code>
 *      ResultDataProvider provider = new ResultDataProvider();
 *      provider.setIncrement(1, Calendar.HOUR);
 *      Calendar cal = Calendar.getInstance();
 *      cal.set(2004, Calendar.JANUARY, 1, 0, 0, 0);
 *      cal.set(Calendar.MILLISECOND, 0);
 *      provider.setPeriod(cal, 5, Calendar.HOUR);
 *      provider.checkSequence(System.out);
 *  </code>
 *  ergibt:
 *      01.01.2004 00:00:00.000
 *      01.01.2004 01:00:00.000
 *      01.01.2004 02:00:00.000
 *      01.01.2004 03:00:00.000
 *      01.01.2004 04:00:00.000
 *  </pre>
 *
 * <p>Die Daten werden nicht unbedingt f&uuml;r den gesamten zu bearbeitenen Zeitraum aus der
 * Datenbank geholt, sondern abschnittsweise. Die L&auml;nge des Zeitbereiches, f&uuml;r den die Daten
 * mit einmal aus dem Archiv gelesen wird, ergibt sich aus (ReadIntervallLength * IncrementUnit).</p>
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: ResultDataProvider.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class ResultDataProvider extends java.lang.Object {
    private static int                                    ArsType_LOGOS      = 1;
    private static int                                    ArsType_MARZ       = 2;
    private static int                                    ArsType_UNASSIGNED = 0;

	/** DebugLogger fuer Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();
    private static SimpleDateFormat                       DATE_FORMAT        = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss.SSS ");

	//
	private static final String CRLF = System.getProperty("line.separator");

    //~ FELDER ================================================================

    int                           _IncrementValue         = 1;
    private boolean               _isAchslastExportEF1    = false;
    ArrayList<SystemObject>       _ListSystemObject       = new ArrayList<SystemObject>(6);
    ArrayList<DataIdentification> _ListDataIdentification = new ArrayList<DataIdentification>(6);
    ArrayList<DataDescription>    _DataDescriptionList    = new ArrayList<DataDescription>(6);
    int                           __ArsType               = ArsType_UNASSIGNED;
    int                           _PeriodLength           = 1;

    /**
     * wird zwischendurch mal auf den gerade bearbeiteten Zeitpunkt gesetzt um
     * einen Fortschritt zu ermitteln.
     */
    Calendar                     _ProcessingCalendar  = Calendar.getInstance();
    boolean                      _IsNotInterrupted    = true;
    int                          _ReadIntervallLength = 24;
    ArrayList<ResultDataChannel> _DataChannelList     = new ArrayList<ResultDataChannel>(4);
    int                          _IncrementUnit;
    int                          _PeriodLengthUnit;
    Calendar                     _PeriodStart;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public ResultDataProvider() {
        if (__ArsType == ArsType_UNASSIGNED) {
            try {
                NetzClientDavConnection.getInstance().getArchive();
                __ArsType = ArsType_MARZ;
            }
            catch (Exception ex) {
                __ArsType = ArsType_LOGOS;
            }
        }
    }

    //~ METHODEN ==============================================================

    /**
     * meldet einen Channel an.
     *
     * @param dataChannel ResultDataChannel
     * @throws ChannelException Exception.
     */
    public void addChannel(ResultDataChannel dataChannel) throws ChannelException {
        this._DataChannelList.add(dataChannel);
    }

    /**
     * f&uuml;gt eine Kombination aus SystemObject und DataDescription zur Liste
     * der Kombinationen hinzu, dessen
     * Daten verarbeitet werden soll. Die Daten der Kombination werden
     * entsprechend der Reihenfolge in der Liste zum ResultData[] zusammengefasst
     * und an die Channels &uuml;bergeben.
     *
     * @param so SystemObject
     * @param dataDescription DataDescription
     */
    public void addDataCombination(SystemObject so, DataDescription dataDescription) {
        this._ListSystemObject.add(so);
        this._DataDescriptionList.add(dataDescription);
        this._ListDataIdentification.add(new DataIdentification(so, dataDescription));
    }

    /**
     * berechnet den gesamten zu bearbeitenden Zeitraum in Minuten, um ein
     * Mass f&uuml;r den Fortschritt zu haben.
     *
     * @param calPeriodStart ToDo.
     * @param calPeriodEnd ToDo.
     * @return der insgesamt zu bearbeitende Zeitraum.
     */
    public static int calculateTotalPeriodInMinutes(Calendar calPeriodStart, Calendar calPeriodEnd) {
        long lTimeDiffInMillis = calPeriodEnd.getTimeInMillis() - calPeriodStart.getTimeInMillis();

        return (int) (lTimeDiffInMillis / 1000 / 60);
    }

    /**
     * berechnet den gesamten zu bearbeitenden Zeitraum in Minuten, um ein
     * Mass f&uuml;r den Fortschritt zu haben.
     * @param calPeriodStart ToDo.
     * @param periodLengthUnit ToDo.
     * @param periodLength ToDo.
     * @return der insgesamt zu bearbeitende Zeitraum.
     */
    public static int calculateTotalPeriodInMinutes(Calendar calPeriodStart, int periodLengthUnit, int periodLength) {
        Calendar calEnd = (Calendar) calPeriodStart.clone();

        calEnd.add(periodLengthUnit, periodLength);

        long lTimeDiffInMillis = calEnd.getTimeInMillis() - calPeriodStart.getTimeInMillis();

        return (int) (lTimeDiffInMillis / 1000 / 60);
    }

    /**
     * erm&ouml;glicht das Pr&uuml;fen der Zeitpunkt, die sich aus der Periode und
     * Schrittweise ergeben.
     *
     * @param out PrintStream
     */
    public void checkSequence(PrintStream out) {
        Calendar endCalendar = (Calendar) _PeriodStart.clone();

        endCalendar.add(_PeriodLengthUnit, _PeriodLength);

        Calendar rollingStartCalendar = (Calendar) _PeriodStart.clone();
        Calendar rollingEndCalendar   = (Calendar) _PeriodStart.clone();

        debug.finer(String.format("startCal: %s", DATE_FORMAT.format(rollingStartCalendar.getTime())));
	    debug.finer(String.format("endCal:   %s", DATE_FORMAT.format(rollingEndCalendar.getTime())));

        while (rollingEndCalendar.before(endCalendar)) {
            rollingEndCalendar.add(_IncrementUnit, _ReadIntervallLength);
	        debug.finer(String.format("endCal: %s", DATE_FORMAT.format(rollingEndCalendar.getTime())));

            if (rollingEndCalendar.after(endCalendar)) {
                rollingEndCalendar = endCalendar;
            }

            for (; rollingStartCalendar.before(rollingEndCalendar); rollingStartCalendar.add(_IncrementUnit, _IncrementValue)) {
	            out.printf("   %s%n", DATE_FORMAT.format(rollingStartCalendar.getTime()));
            }
        }
    }

    /**
     * l&ouml;scht alle Channel.
     */
    public void clearChannels() {
        this._DataChannelList.clear();
    }

    /**
     * l&ouml;scht alle DatenKombinationen.
     */
    public void clearDataCombinations() {
        this._ListSystemObject.clear();
        this._DataDescriptionList.clear();
        this._ListDataIdentification.clear();
    }

    //~ GET METHODEN ==========================================================

    /**
     * gibt den bereits bearbeiteten Zeitraum in Minuten zur&uuml;ck, um ein Mass
     * f&uuml;r den Fortschritt zu haben.
     *
     * @return der insgesamt zu bearbeitende Zeitraum.
     */
    public int getProgressedPeriodInMinutes() {
        long lTimeDiffInMillis = _ProcessingCalendar.getTimeInMillis() - _PeriodStart.getTimeInMillis();

        return (int) (lTimeDiffInMillis / 1000 / 60);
    }

    public final static String getTimeUnit(int unit) {
        switch (unit) {
        case 1 : {
            return "Year";
        }

        case 2 : {
            return "Month";
        }

        case 5 : {
            return "Day";
        }

        case 10 : {
            return "Hour";
        }

        case 11 : {
            return "HourOfDay";
        }

        case 12 : {
            return "Minute";
        }

        case 13 : {
            return "Second";
        }

        case 14 : {
            return "Millisecond";
        }

        case 6 : {
            return "DayOfYear";
        }

        case 7 : {
            return "DayOfWeek";
        }

        case 8 : {
            return "DayOfWeekInMonth";
        }

        case 4 : {
            return "WeekOfMonth";
        }

        case 3 : {
            return "WeekOfYear";
        }

        default : {
            return "unbekannt";
        }
        }
    }

    private Dataset getValidResultData(Dataset[] rsArray, Calendar calendar) {
        Dataset retValue = null;

        if (rsArray != null) {
            for (int i = rsArray.length - 1; ((retValue == null) && (i >= 0)); i--) {
                if ((calendar.getTimeInMillis() == rsArray[i].getDataTime()) && (rsArray[i].getDataType() == DataState.DATA)) {
                    retValue = rsArray[i];
                }
            }
        }

        return retValue;
    }

    //~ METHODEN ==============================================================

    public void interrupt() {
        _IsNotInterrupted = false;
    }

    /**
     * Für alle Zeitpunkte werden die Daten der Kombinationen aus SystemObject und DataDescription
     * aus dem Archiv gelesen und an die die angemeldeten Channels &uuml;bergeben.
     *
     * @throws DataProviderException Exception.
     * @throws ChannelException Exception.
     */
    public void processData() throws ChannelException, DataProviderException {
        // System.out.println("ResultDataProvider.processData Daten aus dem Archiv lesen ....");

	    // siehe Beschreibung zu BAStExportSteuerung._davArchivDatenverteiler !
	    ClientDavInterface davConn = NetzClientDavConnection.getInstanceArchiv();

        // checkSequence(System.err);
        int diCount = _ListDataIdentification.size();

        // ArchiveData[][] rsArrays = new ArchiveData[soCount][];
        HashMap<Long, ArchiveData[]> teilBereichsContainer = new HashMap<Long, ArchiveData[]>(_ReadIntervallLength);
        Calendar                     bereichsEnde          = (Calendar) _PeriodStart.clone();

        bereichsEnde.add(_PeriodLengthUnit, _PeriodLength);
        _ProcessingCalendar.setTimeInMillis(_PeriodStart.getTimeInMillis());

        Calendar calTeilBereichAnfang = (Calendar) _PeriodStart.clone();
        Calendar calTeilBereichEnde   = (Calendar) _PeriodStart.clone();
        Calendar cal                  = (Calendar) _PeriodStart.clone();

	    debug.info(this.toString());
	    debug.finer(String.format("Anfrage von/bis = %s  %s", DATE_FORMAT.format(new java.util.Date(_PeriodStart.getTimeInMillis())), DATE_FORMAT.format(new java.util.Date(bereichsEnde.getTimeInMillis()))));

        while ((calTeilBereichEnde.before(bereichsEnde)) && _IsNotInterrupted) {
            calTeilBereichEnde.add(_IncrementUnit, _ReadIntervallLength);

            if (calTeilBereichEnde.after(bereichsEnde)) {
                calTeilBereichEnde = bereichsEnde;
            }

            teilBereichsContainer.clear();
            cal.setTime(calTeilBereichAnfang.getTime());

            if (!_isAchslastExportEF1) {
                while (cal.before(calTeilBereichEnde)) {
                    teilBereichsContainer.put(cal.getTimeInMillis(), new ArchiveData[_DataDescriptionList.size()]);
                    cal.add(_IncrementUnit, _IncrementValue);

                    if (cal.after(calTeilBereichEnde)) {
                        cal.setTimeInMillis(calTeilBereichEnde.getTimeInMillis());

                        // teilBereichsContainer.put(cal.getTimeInMillis(), new ArchiveData[_DataDescriptionList.size()]);
                    }
                }
            }

            // logger.info("TeilBereich ");
            // ArrayList<Long> timeList = new ArrayList<Long>(teilBereichsContainer.keySet());
            // Collections.sort(timeList);
            // for (Long lTime : timeList) {
            // logger.info("    %1$td.%1$tm.%1$ty %1$tH:%1$tM", new Date(lTime));
            // }
	        debug.info(String.format("TeilAnfrage: %1$td.%1$tm.%1$ty %1$tH:%1$tM  %2$td.%2$tm.%2$ty %2$tH:%2$tM", calTeilBereichAnfang.getTime(), calTeilBereichEnde.getTime()));

	        //
	        long lTeilBereichEnde   = calTeilBereichEnde.getTimeInMillis();
	        long lTeilBereichAnfang = calTeilBereichAnfang.getTimeInMillis();

            for (int diNr = 0; diNr < diCount; diNr++) {
                DataIdentification dataIdentification = _ListDataIdentification.get(diNr);
                String             soPid              = dataIdentification.getObject().getPid();
                String             atgPid             = dataIdentification.getDataDescription().getAttributeGroup().getPid();
                String             aspPid             = dataIdentification.getDataDescription().getAspect().getPid();

                try {
                    ArchiveDataStream arsDataStream = ArchiveRequestHelper.getStream(davConn, soPid, atgPid, aspPid, lTeilBereichAnfang, lTeilBereichEnde);
                    ArchiveData       arsData       = arsDataStream.take();

                    while (arsData != null) {
                        if ((arsData.getDataTime() >= lTeilBereichAnfang) && (arsData.getDataTime() < lTeilBereichEnde) && (arsData.getDataType().equals(DataState.DATA))) {
                            if (_isAchslastExportEF1) {
                                ArchiveData[] arsDataAr = new ArchiveData[1];

                                if (arsData != null) {
                                    arsDataAr[0] = arsData;
                                    teilBereichsContainer.put(arsData.getDataTime(), arsDataAr);

                                } else {

                                }
                            } else {
                                ArchiveData[] arsDataArray = teilBereichsContainer.get(arsData.getDataTime());

                                if (arsDataArray == null) {

                                    // throw new Exception("kein Speicherplatz für "+new Date(arsData.getDataTime()));
	                                debug.error(String.format("Daten mit Zeitstempel %s erhalten, dem kein Speicherplatz zugeordnet ist (Zeitpunkt kein gültiger Intervallbeginn).", new Date(arsData.getDataTime())));
                                } else {
                                    if (arsDataArray[diNr] == null) {
                                        arsDataArray[diNr] = arsData;
                                    } else {
                                        if (arsData.getDataIndex() > arsDataArray[diNr].getDataIndex()) {
                                            arsDataArray[diNr] = arsData;
                                        }
                                    }
                                }
                            }
                        }

                        arsData = arsDataStream.take();
                    }
                }
                catch (Exception e) {

                    // DataIdentification di = _ListDataIdentification.get(diNr);
                    e.printStackTrace();

                    throw new DataProviderException("Fehler beim Lesen " + soPid + ':' + atgPid + ':' + aspPid, e);
                }
            }

            cal.setTime(calTeilBereichAnfang.getTime());

            if (_isAchslastExportEF1) {
                Calendar aktuelleZeit = Calendar.getInstance();
	            ArrayList<Long> sortierteZeiten = new ArrayList<Long>();
	            sortierteZeiten.addAll(teilBereichsContainer.keySet());
	            Collections.sort(sortierteZeiten);
	            // für jeden Datensatz einmal processRecord aufrufen
	            for (long zeit : sortierteZeiten) {
                    aktuelleZeit.setTimeInMillis(zeit);

                    for (ResultDataChannel channel : _DataChannelList) {
                        channel.processRecord(teilBereichsContainer.get(zeit), aktuelleZeit);
                    }

                   
                }
            } else {
                while (cal.before(calTeilBereichEnde)) {
                    ArchiveData[] arsDataArray = teilBereichsContainer.get(cal.getTimeInMillis());

	                //
	                StringBuilder sb = new StringBuilder();

	                sb.append((arsDataArray[0] == null) ? "null" : DebugHelper.getDataIndexAsString(arsDataArray[0].getDataIndex()));

	                for (int i = 1; i < arsDataArray.length; i++) {
		                sb.append(", ").append((arsDataArray[0] == null) ? "null" : DebugHelper.getDataIndexAsString(arsDataArray[0].getDataIndex()));
	                }

	                debug.finer(String.format("   processRecord  %1$td.%1$tm.%1$ty %1$tH:%1$tM %2$s", cal, sb));

	                //
                    for (ResultDataChannel channel : _DataChannelList) {
                        channel.processRecord(arsDataArray, cal);
                    }

                    cal.add(_IncrementUnit, _IncrementValue);
                }
            }

            calTeilBereichAnfang.setTime(cal.getTime());
        }
    }

    /**
     * Für alle Zeitpunkte werden die Daten der Kombinationen aus SystemObject und DataDescription
     * aus dem Archiv gelesen und an die die angemeldeten Channels &uuml;bergeben.
     *
     * @throws DataProviderException Exception.
     * @throws ChannelException Exception.
     */

    // public void processData() throws ConfigurationException, DataNotSubscribedException, ChannelException {
    public void processDataOld() throws ChannelException, DataProviderException {
	    ClientDavInterface davConn     = NetzClientDavConnection.getInstance();
        int                      soCount     = _ListSystemObject.size();
        ArchiveData[][]          rsArrays    = new ArchiveData[soCount][];
        Calendar                 endCalendar = (Calendar) _PeriodStart.clone();

        endCalendar.add(_PeriodLengthUnit, _PeriodLength);
        _ProcessingCalendar.setTimeInMillis(_PeriodStart.getTimeInMillis());

        Calendar rollingStartCalendar = (Calendar) _PeriodStart.clone();
        Calendar rollingEndCalendar   = (Calendar) _PeriodStart.clone();

	    debug.info(this.toString());

        try {
	        debug.finer(String.format("Anfrage von/bis = %s  %s", DATE_FORMAT.format(new java.util.Date(_PeriodStart.getTimeInMillis())), DATE_FORMAT.format(new java.util.Date(endCalendar.getTimeInMillis()))));

            while ((rollingEndCalendar.before(endCalendar)) && _IsNotInterrupted) {
                rollingEndCalendar.add(_IncrementUnit, _ReadIntervallLength);

                if (rollingEndCalendar.after(endCalendar)) {
                    rollingEndCalendar = endCalendar;
                }

	            debug.info(String.format("TeilAnfrage: %s  %s", DATE_FORMAT.format(rollingStartCalendar.getTime()), DATE_FORMAT.format(rollingEndCalendar.getTime())));

                for (int i = 0; i < soCount; i++) {
                    String        soPid       = _ListSystemObject.get(i).getPid();
                    ArchiveData[] rsDataArray = null;

                    try {
                        DataDescription dataDescr = (DataDescription) _DataDescriptionList.get(i);
                        String          atgPid    = dataDescr.getAttributeGroup().getPid();
                        String          aspPid    = dataDescr.getAspect().getPid();

                        System.gc();

                        Set<ArchiveData> arsData = ArchiveRequestHelper.getArchivData(davConn, soPid, atgPid, aspPid, rollingStartCalendar.getTimeInMillis(), rollingEndCalendar.getTimeInMillis() - 10);

                        rsDataArray = arsData.toArray(new ArchiveData[arsData.size()]);
	                    debug.info(String.format("ResultCount=%s", arsData.size()));

                        // test.ResultDataTest.checkMem("rsdp");
                    }
                    catch (Exception e) {
                        DataDescription dataDescr = (DataDescription) _DataDescriptionList.get(i);

                        e.printStackTrace();

                        throw new DataProviderException("Fehler beim Lesen " + soPid + ':' + dataDescr.getAttributeGroup() + ':' + dataDescr.getAspect() + ':' + dataDescr.getSimulationVariant(), e);
                    }

                    if ((rsDataArray != null) && (rsDataArray.length > 0)) {
                        rsArrays[i] = rsDataArray;
                    } else {
                        rsArrays[i] = null;
                    }
                }

                _ProcessingCalendar.setTimeInMillis(rollingEndCalendar.getTimeInMillis());

                for (; (rollingStartCalendar.before(rollingEndCalendar) && _IsNotInterrupted); rollingStartCalendar.add(_IncrementUnit, _IncrementValue)) {
                    Dataset[] rsArrayAtTime = new Dataset[soCount];

                    for (int i = 0; i < soCount; i++) {
                        rsArrayAtTime[i] = getValidResultData(rsArrays[i], rollingStartCalendar);
                    }

                    if ((rsArrayAtTime[0] != null) && (rsArrayAtTime[0].getDataTime() != rollingStartCalendar.getTimeInMillis())) {
	                    debug.error("Zeitstempel der Daten stimmt nicht mit dem übergebenen Calendar-Object überein.");

                        throw new IllegalStateException("Zeitstempel der Daten stimmt nicht mit dem übergebenen Calendar-Object überein.");
                    }

                    for (int channelNr = 0; channelNr < _DataChannelList.size(); channelNr++) {
                        ((ResultDataChannel) _DataChannelList.get(channelNr)).processRecord(rsArrayAtTime, rollingStartCalendar);
                    }
                }

                _ProcessingCalendar.setTimeInMillis(rollingEndCalendar.getTimeInMillis());
            }
        }
        catch (DataNotSubscribedException ex) {
            throw new DataProviderException("Datenhaltung ist nicht angemeldet.", ex);
        }
    }

    //~ SET METHODEN ==========================================================

    public void setAchslastExportEF1(boolean achslastExportEF1) {
        _isAchslastExportEF1 = achslastExportEF1;
    }

    /**
     * setzt die Schrittweite zur Berechnung der Zeitpunkte. Der erste Zeitpunkt
     * ist gleich dem Anfangszeitpunktes des Zeitraumes.
     *
     * @param incrementValue Wert der Schrittweite
     * @param incrementUnit Einhet der Schrittweite
     */
    public void setIncrement(int incrementValue, int incrementUnit) {
        this._IncrementValue = incrementValue;
        this._IncrementUnit  = incrementUnit;
    }

    /**
     * setzt den Zeitraum f&uuml;r den die Zeitpunkte erzeugt werden. Der erste
     * Zeitpunkt ist gleich dem Anfangszeitpunkt.
     *
     * @param periodStart Anfang der Zeitraumes
     * @param periodLength Wert der L&auml;nge der Zeitraumes.
     * @param unit Einheit der Zeitraumlänge
     */
    public void setPeriod(Calendar periodStart, int periodLength, int unit) {
        this._PeriodStart      = periodStart;
        this._PeriodLength     = periodLength;
        this._PeriodLengthUnit = unit;
    }

    /**
     * <p>setzt die ReadIntervallLength. (ReadIntervallLength *  IncrementUnit)
     * ergibt die Zeitspanne, f&uuml;r die Daten vom Archiv geholt werden.</p>
     *
     * @param readIntervallLength int
     */
    public void setReadIntervallLength(int readIntervallLength) {
        this._ReadIntervallLength = readIntervallLength;
    }

    //~ METHODEN ==============================================================

    public String toString() {
        StringBuffer sb = new StringBuffer();

        sb.append("ResultDataProvider: (PeriodStart: ");
        sb.append(DATE_FORMAT.format(_PeriodStart.getTime()));
        sb.append(" - PeriodenLaenge: ").append(_PeriodLength).append(' ').append(getTimeUnit(_PeriodLengthUnit));
        sb.append(" - Increment: ").append(_IncrementValue).append(' ').append(getTimeUnit(_IncrementUnit));
        sb.append(" - PeriodenLaenge: ").append(_PeriodLength).append(' ').append(getTimeUnit(_PeriodLengthUnit));

        Calendar calEnde = (Calendar) _PeriodStart.clone();

        calEnde.add(_PeriodLengthUnit, _PeriodLength);
        sb.append(" - Ende: ").append(DATE_FORMAT.format(calEnde.getTime()));
        sb.append(CRLF);
        sb.append(" - DataCombinations: ");

        if (_ListSystemObject == null) {
            sb.append("null");
        } else {
            for (int i = 0; i < _ListSystemObject.size(); i++) {
                if (i > 0) {
                    sb.append(", ");
                }

                sb.append(_ListSystemObject.get(i).getPid());
                sb.append(':').append(_DataDescriptionList.get(i).getAttributeGroup().getPid());
                sb.append(':').append(_DataDescriptionList.get(i).getAspect().getPid());
            }
        }

        sb.append(')');

        return sb.toString();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 24.09.10
