/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;
/**
 * History:
 * 2007-10-22 [KS]  Sortierfunktion für ConfigurationObjectWrapper[] nach Name,
 *                  Pid und TreeString
 *
 * Revision 1.4  2008/09/01 08:17:17  Schawaller
 * Sortierung ByName überarbeitet
 *
 */

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.inovat.kex.bast.common.*;

/**
 * BasisKlasse f&uuml;r Klassen, die ConfigurationObjects eines bestimmten
 * ConfigurationObjectTypes wrappen.
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: ConfigurationObjectWrapper.java 442 2008-11-10 06:59:32Z Schawaller $
 */
abstract public class ConfigurationObjectWrapper
    extends java.lang.Object {

  public static ConfigurationObjectType __TYP_ANSCHLUSSPUNKT;
  public static ConfigurationObjectType __TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER;
  public static AttributeGroup __ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER;

  /** Vergleichoperator für Sortierung der Konfigurationsobjekte nach dem Namen */
  private static Comparator<? super ConfigurationObjectWrapper> __ByNameSorter =
      new Comparator<ConfigurationObjectWrapper>()
  {
    public boolean equals(Object obj) {
      return (obj==this);
    }
    public int compare(ConfigurationObjectWrapper object01, ConfigurationObjectWrapper object02) {
      String s1 = object01.getName();
      String s2 = object02.getName();
      if ((s1 != null) && (s2 != null)) {
        return (s1.compareToIgnoreCase(s2));
      }
      if (s1 == null) {
        return 1;
      }
      return -1;
    }
  };

  /** Vergleichoperator für Sortierung der Konfigurationsobjekte nach der Pid */
  private static Comparator<? super ConfigurationObjectWrapper> __ByPidSorter =
      new Comparator<ConfigurationObjectWrapper>()
  {
    public boolean equals(Object obj) {
      return (obj==this);
    }
    public int compare(ConfigurationObjectWrapper object01, ConfigurationObjectWrapper object02) {
      return (object01.getPID().compareToIgnoreCase(object02.getPID()));
    }
  };
  /** Vergleichoperator für Sortierung der Konfigurationsobjekte nach der Pid */
  private static Comparator<? super ConfigurationObjectWrapper> __ByTreeStringSorter =
      new Comparator<ConfigurationObjectWrapper>()
  {
    public boolean equals(Object obj) {
      return (obj==this);
    }
    public int compare(ConfigurationObjectWrapper object01, ConfigurationObjectWrapper object02) {
      return (object01.getTreeString().compareToIgnoreCase(object02.getTreeString()));
    }
  };

  /**
   * das gewrappte ConfigurationObject.
   */
  protected ConfigurationObject _ConfigurationObject;

  /**
   * gibt die PID des ConfigurationObjectes zur&uuml;ck.
   *
   * @return die PID
   */
  public final String getPID() {
    return _ConfigurationObject.getPid();
  }

  /**
   * gibt den Namen des ConfigurationObjectes zur&uuml;ck.
   *
   * @return den Namen
   */
  public final String getName() {
    return _ConfigurationObject.getName();
  }

  /**
   * gibt das gewrappte ConfigurationObjectes zur&uuml;ck.
   *
   * @return die PID
   */
  public final ConfigurationObject getConfigurationObject() {
    return _ConfigurationObject;
  }

  /**
   * Implementationen m&uuml;ssen den ConfigurationObjectType zur&uuml;ckgeben,
   * den die gewrappten ConfigurationObjecte haben.
   *
   * @return ConfigurationObjectType der gewrappten ConfigurationObjecte.
   */
  abstract ConfigurationObjectType getType();


  /**
   * gibt einen String zum Anzeigen in einem Tree zur&uuml;ck.
   * 
   * @return Text zur Anzeige im Baum
   */
  abstract public String getTreeString();


  /**
   * gibt einen String zum Anzeigen als ein ToolTip zur&uuml;ck.
   * 
   * @return Tooltipp
   */
  public String getToolTipText() {
    return null;
  }

  /**
   * Implemtationen sollen hier die konfigurierenden AttributGruppen und die
   * zugeh&ouml;rigen Mengen lesen.
   *
   * @param confObject ConfigurationObject
   * @throws DataModellMismatchException Exception.
   */
  abstract void initData(ConfigurationObject confObject) throws DataModellMismatchException;

  /**
   * erzeugt eine Instance auf Basis der PID des zu wrappenden Objectes.
   *
   * @param pid die PID des zu wrappenen Objectes
   * @throws DataModellMismatchException wenn der Typ des ConfigurationsObject
   *   nicht durch die Klasse gewrappt werden kann oder das Object mit der
   *   PID kein ConfigurationObject ist.
   * @throws DataModellObjectNotFoundException wenn kein Object mit dieser PID
   *   existiert.
   */
  protected ConfigurationObjectWrapper(String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
    SystemObject obj = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
    if (obj == null) {
      throw new DataModellObjectNotFoundException("Objekt " + pid + " konnte nicht gefunden werden", pid);
    }
    if (! (obj instanceof ConfigurationObject)) {
      throw new DataModellMismatchException("Objekt " + pid + " ist kein ConfigurationObject", pid);
    }
    _ConfigurationObject = (ConfigurationObject) obj;

    if (!_ConfigurationObject.isOfType(this.getType())) {
      throw new DataModellMismatchException("Objekt " + pid + " ist nicht vom typ " + this.getType(), pid);
    }
    else {
      initData(_ConfigurationObject);
    }
  }

  /**
   * erzeugt eine Instance, die das angegebene Object wrappt.
   *
   * @param confObj zu wreppenes ConfigurationObject
   * @throws DataModellMismatchException Exception.
   */
  protected ConfigurationObjectWrapper(ConfigurationObject confObj) throws DataModellMismatchException {
    _ConfigurationObject = confObj;
    if (confObj.isOfType(this.getType())) {
      initData(confObj);
    }
    else {
      throw new DataModellMismatchException("Objekt " + confObj.getPid() + " ist nicht vom typ " + this.getType(), confObj.getPid());
    }
  }


  /**
   * gibt einen String "ObjectType - ObjectName" zur&uuml;ck.
   *
   * @return String
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();
    try {
      sb.append(this.getType().getName() + " - ").append(this.getName());
      return sb.toString();
    }
    catch (Exception e) {
      return super.toString();
    }
  }

  /**
   * gibt die KommunikationsPartner zur&uuml;ck, die &uuml;ber die
   * AnschlussPunkte erreichbar sind.
   *
   * @param anschlussPunktGeraeteListe List
   * @return Array der KommunikationsPartner.
   */
  protected static List<SystemObject> getKommunikationPartner(List<SystemObject> anschlussPunktGeraeteListe) {

    if (__TYP_ANSCHLUSSPUNKT ==  null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __TYP_ANSCHLUSSPUNKT = (ConfigurationObjectType)model.getType("typ.anschlussPunkt");
      __TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = (ConfigurationObjectType)model.getType("typ.anschlussPunktKommunikationsPartner");
      __ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = model.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
    }
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();

    if ( (anschlussPunktGeraeteListe != null) && (anschlussPunktGeraeteListe.size() > 0)) {
      for (int apgNr = 0; (apgNr < anschlussPunktGeraeteListe.size()); apgNr++) {
        try {
          ConfigurationObject apgObject = (ConfigurationObject)anschlussPunktGeraeteListe.get(apgNr);
          if ((apgObject!=null) && (apgObject.isOfType(__TYP_ANSCHLUSSPUNKT))) {
            ObjectSet apkpOS = apgObject.getObjectSet("AnschlussPunkteKommunikationsPartner");
            if (apkpOS != null) {
              List<SystemObject> listAPKP = apkpOS.getElements();
              if ( (listAPKP != null) && (listAPKP.size() > 0)) {
                for (int apkpNr = 0; apkpNr < listAPKP.size(); apkpNr++) {
                  ConfigurationObject apkpObject = (ConfigurationObject)listAPKP.get(apkpNr);
                  if ((apkpObject!=null) && (apkpObject.isOfType(__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER))) {
                    Data data = apkpObject.getConfigurationData(__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER);
                    Data.ReferenceValue refValue = data.getReferenceValue("KommunikationsPartner");
                    if (refValue!=null) {
                      SystemObject so = refValue.getSystemObject();
                      list.add(so);
                    }
                  }
                }
              }
            }
          }
        }
        catch (Exception e) {
          e.printStackTrace();
        }
      }
    }
    return list;
  }


  protected static List<SystemObject> getKommunikationPartner(ConfigurationObject apgObject) {

    if (__TYP_ANSCHLUSSPUNKT ==  null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __TYP_ANSCHLUSSPUNKT = (ConfigurationObjectType)model.getType("typ.anschlussPunkt");
      __TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = (ConfigurationObjectType)model.getType("typ.anschlussPunktKommunikationsPartner");
      __ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = model.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");

      List<SystemObject> list = __TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER.getObjects();
      model.getConfigurationData(list, __ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER);
    }
    ArrayList<SystemObject> list = new ArrayList<SystemObject>();

    try {
      if ((apgObject!=null) && (apgObject.isOfType(__TYP_ANSCHLUSSPUNKT))) {
        ObjectSet apkpOS = apgObject.getObjectSet("AnschlussPunkteKommunikationsPartner");
        if (apkpOS != null) {
          List<SystemObject> listAPKP = apkpOS.getElements();
          if ( (listAPKP != null) && (listAPKP.size() > 0)) {
            for (int apkpNr = 0; apkpNr < listAPKP.size(); apkpNr++) {
              ConfigurationObject apkpObject = (ConfigurationObject)listAPKP.get(apkpNr);
              if ((apkpObject!=null) && (apkpObject.isOfType(__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER))) {
                Data data = apkpObject.getConfigurationData(__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER);
                Data.ReferenceValue refValue = data.getReferenceValue("KommunikationsPartner");
                if (refValue!=null) {
                  SystemObject so = refValue.getSystemObject();
                  list.add(so);
                }
              }
            }
          }
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return list;
  }

  /**
   * <p>Sortiert das Array von ConfigurationObjectWrapper Objekten nach Namen.</p>
   * @param objectArray ToDo.
   */
  public static final void sortByName(ConfigurationObjectWrapper[] objectArray) {
    Arrays.sort(objectArray, __ByNameSorter);
  }

  /**
   * <p>Sortiert das Array von ConfigurationObjectWrapper Objekten nach PID.</p>
   * @param objectArray ToDo.
   */
  public static final void sortByPid(ConfigurationObjectWrapper[] objectArray) {
    Arrays.sort(objectArray, __ByPidSorter);
  }

  /**
   * <p>Sortiert das Array von ConfigurationObjectWrapper Objekten nach den
   * Tree-Strings.</p>
   * @param objectArray ToDo.
   */
  public static final void sortByTreeString(ConfigurationObjectWrapper[] objectArray) {
    Arrays.sort(objectArray, __ByTreeStringSorter);
  }

}
