/*
 * Copyright (c) 2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.pat.sftptest.SFTPFunktionen
 *
 * de.inovat.pat.sftptest.SFTPFunktionen is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.pat.sftptest.SFTPFunktionen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.sftptest.SFTPFunktionen; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */

package de.inovat.kex.bast.serverfunc;

//~ NICHT JDK IMPORTE =========================================================

import com.jcraft.jsch.*;

//~ JDK IMPORTE ===============================================================

import java.io.File;

import java.util.*;

//~ KLASSEN ===================================================================

/**
 * Klasse stellt statischen SFTP-Funktionen zu Verfügung.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class SFTPFunktionen {
    private static final String SFTP = "sftp";

    //~ METHODEN ==============================================================

    /**
     * Ermittle alle Dateiennamen im angegebenen SFTP-Ordner.
     *
     * @param session Session.
     * @param sftpOrdner SFTP-Ordner.
     *
     * @return Liste der Dateiennamen im angegebenen SFTP-Ordner
     * oder null, falls nicht ermittelbar ist.
     */
    public static List<String> ermittleDateienImSFTPOrdner(Session session, String sftpOrdner) {
        List<String> listeDateien = new ArrayList<String>();
        ChannelSftp  sftpChannel  = null;

        try {

            // Öffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // Ermittle alle Dateien im SFTP-Ordner:
            Vector vOrdnerInhalt;

            if (sftpOrdner.isEmpty()) {
                vOrdnerInhalt = sftpChannel.ls("/");
            } else {
                vOrdnerInhalt = sftpChannel.ls(sftpOrdner);
            }

            if (vOrdnerInhalt != null) {
                for (Object obj : vOrdnerInhalt) {
                    if (obj instanceof ChannelSftp.LsEntry) {
                        ChannelSftp.LsEntry lse = (ChannelSftp.LsEntry) obj;

                        if (!lse.getAttrs().isDir()) {
                            listeDateien.add(lse.getFilename());
                        }
                    }
                }
            }
        }
        catch (Exception e) {
            listeDateien = null;
        }

        schliesseSftpKanal(sftpChannel);

        return listeDateien;
    }

    /**
     * Schliesse den SFTP-Kanal.
     *
     * @param sftpChannel ChannelSftp
     */
    public static void schliesseSftpKanal(ChannelSftp sftpChannel) {

        // Versuche den Kanal zu schließen:
        try {
            if (sftpChannel != null) {
                sftpChannel.disconnect();
            }
        }
        catch (Exception ignored) {}
    }

    /**
     * Übertrage lokale Datei auf SFTP-Server.
     * <p>
     * Lokale Datei wird zuerst in SFTP-Lock-Datei übertragen.
     * Falls die Größen der lokalen und übertragenen Dateien gleich sind,
     * wird die SFTP-Datei umbenannt.
     *
     * @param session SFTP-Session.
     * @param dateiLokal Die lokale zu übertragende Datei.
     * @param dateiSftp SFTP-Datei als Ergebnis der Übertragung.
     * @param dateiSftpLock SFTP-Lock-Datei, wohin die lokale Datei kopiert werden soll.
     *
     * @return Exception, falls beim Übertragen _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception uebertrageLokaleDateiAufSFTPServer(Session session, File dateiLokal, String dateiSftp, String dateiSftpLock) {
        ChannelSftp sftpChannel = null;

        try {

            // ---------------------------------------------------------
            // Öffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // Übertrage lokale Datei auf SFTPServer in SFTP-Lock-Datei:
            try {
                sftpChannel.put(dateiLokal.getAbsolutePath(), dateiSftpLock);
            }
            catch (Exception e) {
                Exception e2 = new Exception(String.format("Datei <%s> konnte nicht auf SFTP-Server in <%s> übertragen werden (%s)", dateiLokal.getAbsolutePath(), dateiSftpLock, e.getLocalizedMessage()), e);

                e2.setStackTrace(e.getStackTrace());
                schliesseSftpKanal(sftpChannel);

                return e2;
            }

            //
            // ---------------------------------------------------------
            // Falls die Größen der lokalen und übertragen Dateien gleich sind,
            // umbenenne die Datei auf dem SFTP-Server:
            long sizeLokal = dateiLokal.length();
            long sizeSftp  = sftpChannel.stat(dateiSftpLock).getSize();

            if (sizeSftp == sizeLokal) {
                try {

                    // Lösche Datei (falls existiert):
                    try {
                        sftpChannel.rm(dateiSftp);
                    }
                    catch (Exception ignored) {}

                    // Umbenenne Datei:
                    sftpChannel.rename(dateiSftpLock, dateiSftp);
                    schliesseSftpKanal(sftpChannel);

                    return null;
                }
                catch (Exception e) {
                    Exception e2 = new Exception(String.format("SFTP-Datei <%s> konnte nicht in <%s> umbennant werden", dateiSftpLock, dateiSftp), e);

                    e2.setStackTrace(e.getStackTrace());
                    schliesseSftpKanal(sftpChannel);

                    return e2;
                }
            } else {
                schliesseSftpKanal(sftpChannel);

                return new Exception(String.format("Die übertragene SFTP-Datei <%s> hat falsche Größe (%d statt %d)", dateiSftpLock, sizeSftp, sizeLokal));
            }
        }
        catch (Exception e) {

            // ---------------------------------------------------------
            // Versuche den Kanal zu schließen:
            schliesseSftpKanal(sftpChannel);

            return e;
        }
    }

    /**
     * Übertrage die SFTP-Datei in lokale Datei.
     *
     * @param session Session.
     * @param dateiSftp Die zu übertragende SFTP-Datei.
     * @param dateiLokal Lokale Datei, wohin die SFTP-Datei kopiert werden soll.
     *
     * @return Exception, falls beim Übertragen _Fehler aufgetreten sind,
     *  null - sonst.
     */
    public static Exception uebertrageSFTPDateiInLokaleDatei(Session session, String dateiSftp, String dateiLokal) {
        ChannelSftp sftpChannel = null;

        try {

            // ---------------------------------------------------------
            // Öffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // Übertrage die SFTP-Datei in lokaler Datei:
            try {
                sftpChannel.get(dateiSftp, dateiLokal);
            }
            catch (Exception e) {
                Exception e2 = new Exception(String.format("SFTP-Datei <%s> konnte nicht in lokale Datei <%s> übertragen werden (%s)", dateiSftp, dateiLokal, e.getLocalizedMessage()), e);

                e2.setStackTrace(e.getStackTrace());
                schliesseSftpKanal(sftpChannel);

                return e2;
            }

            // ---------------------------------------------------------
            // Prüfe die Größen:
            long sizeLokal = new File(dateiLokal).length();
            long sizeSftp  = sftpChannel.stat(dateiSftp).getSize();

            if (sizeSftp == sizeLokal) {
                schliesseSftpKanal(sftpChannel);

                return null;
            } else {
                schliesseSftpKanal(sftpChannel);

                return new Exception(String.format("Die übertragene lokale Datei <%s> hat falsche Größe (%d statt %d)", dateiLokal, sizeLokal, sizeSftp));
            }
        }
        catch (Exception e) {

            // ---------------------------------------------------------
            // Versuche den Kanal zu schließen:
            schliesseSftpKanal(sftpChannel);

            return e;
        }
    }

    /**
     * Stelle die Verbindung mit dem SFTP-Server her.
     *
     * @param url SFTP-Host.
     * @param benutzer SFTP-Benutzer.
     * @param passwort SFTP-Passwort (null und leeres String sind erlaubt).
     *
     * @return SFTP-Session.
     *
     * @throws Exception Exception, falls bei der Verbindung _Fehler aufgetreten sind.
     */
    public static Session verbindeMitSFTPServer(String url, String benutzer, String passwort) throws Exception {
        return verbindeMitSFTPServer(url, 22, benutzer, passwort);
    }

    /**
     * Stelle die Verbindung mit dem SFTP-Server her.
     *
     * @param url SFTP-Host.
     * @param port SFTP-Port.
     * @param benutzer SFTP-Benutzer.
     * @param passwort SFTP-Passwort (null und leeres String sind erlaubt).
     *
     * @return SFTP-Session.
     *
     * @throws Exception Exception
     */
    public static Session verbindeMitSFTPServer(String url, int port, String benutzer, String passwort) throws Exception {
        return verbindeMitSFTPServer(url, port, benutzer, passwort, null, null);
    }

    /**
     * Stelle die Verbindung mit dem SFTP-Server her.
     *
     * @param url SFTP-Host.
     * @param port SFTP-Port.
     * @param benutzer SFTP-Benutzer.
     * @param passwort SFTP-Passwort (null und leeres String sind erlaubt).
     * @param prvkeyFile Pfad zu der Datei (private Key) bei der Schlüssel-basierter Authentifizierung
     *               oder null, falls die Authentifizierung nicht verwendet wird.
     * @param prvkeyPassphrase Passwort für private Key, oder null, falls nicht vorhanden ist bzw.
     *                   die Authentifizierung nicht verwendet wird.
     *
     * @return SFTP-Session.
     *
     * @throws Exception Exception
     */
    public static Session verbindeMitSFTPServer(String url, int port, String benutzer, String passwort, String prvkeyFile, String prvkeyPassphrase) throws Exception {
        JSch jsch = new JSch();

        // -----------------------------------------
        // Schlüssel-basierter Authentifizierung
        // -----------------------------------------
        // http://www.jcraft.com/jsch/examples/UserAuthPubKey.java.html
        //
        // Bemerkung: JSch expects the private key to be in OpenSSH format:
        // https://stackoverflow.com/questions/15332120/jsch-invalid-private-key
        // sonst Exception
        // (The cipher 'aes256-cbc' is required, but it is not available)
        // https://stackoverflow.com/questions/17989883/the-cipher-aes256-cbc-is-required-but-it-is-not-available
        //
        if ((prvkeyFile != null)
            &&!prvkeyFile.isEmpty()) {
            jsch.addIdentity(prvkeyFile, prvkeyPassphrase);
        }

        // -----------------------------------------
        // TODO Host-Datei definieren (über Aufrufparameter)
        // -----------------------------------------
        // jsch.setKnownHosts("sftp_inovat\\key\\priv");
        //
        // -----------------------------------------
        // Session öffnen:
        // -----------------------------------------
        Session session = jsch.getSession(benutzer, url, port);

        session.setPassword(passwort);

        /*
        // -----------------------------------------
        // TODO Proxy-Einstellungen:
        // -----------------------------------------
        if (_sftpProxy) {
            // via HTTP proxy:
            session.setProxy(new ProxyHTTP(_sftpProxyHost, _sftpPort));
            // via SOCKS proxy
            session.setProxy(new ProxySOCKS5(_sftpProxyHost, _sftpPort));
        }
        */

        // -----------------------------------------
        // HostKey:
        // -----------------------------------------
        // TODO Überprüfung der HostKeys ausschalten <--- über Aufrufparameter.
        // WICHTIG: es wird automatisch allen Hosts geglaubt, die HostKeys werden nicht überprüft.
        // Durch den Eintrag <config.put("StrictHostKeyChecking", "no")> wird explizit eine Verbindung
        // zu jeden beliebigen Host erlaubt (die Strikte Überprüfung der HostKeys wird unterbinden).
        java.util.Properties config = new java.util.Properties();

        config.put("StrictHostKeyChecking", "no");
        session.setConfig(config);

        // -----------------------------------------
        // Verbindung:
        // -----------------------------------------
        session.connect();

        return session;
    }

    /**
     * Verschiebe bzw. umbenenne SFTP-Datei.
     *
     * @param session Session.
     * @param dateiSftpAlt Die alte SFTP-Datei (mit Path).
     * @param dateiSftpNeu Die neue SFTP-Datei (mit Path).
     *
     * @return Exception, falls beim Verschieben _Fehler aufgetreten sind,
     *  null - sonst.
     */
    public static Exception verschiebeSFTPDatei(Session session, String dateiSftpAlt, String dateiSftpNeu) {
        ChannelSftp sftpChannel = null;

        try {

            // ---------------------------------------------------------
            // Öffne den Kanal:
            Channel channel = session.openChannel(SFTP);

            channel.connect();
            sftpChannel = (ChannelSftp) channel;

            // ---------------------------------------------------------
            // Verschiebe bzw. umbenenne die SFTP-Datei:
            try {
                sftpChannel.rename(dateiSftpAlt, dateiSftpNeu);
                schliesseSftpKanal(sftpChannel);

                return null;
            }
            catch (Exception e) {
                Exception e2 = new Exception(String.format("SFTP-Datei <%s> konnte nicht in <%s> verschiebt/umbenannt werden (%s)", dateiSftpAlt, dateiSftpNeu, e.getLocalizedMessage()), e);

                e2.setStackTrace(e.getStackTrace());
                schliesseSftpKanal(sftpChannel);

                return e2;
            }
        }
        catch (Exception e) {

            // ---------------------------------------------------------
            // Versuche den Kanal zu schließen:
            schliesseSftpKanal(sftpChannel);

            return e;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 02.07.19
